@@ -17,9 +17,8 @@
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 # 
-#----------------------------------------------------------------------------
-#
-# $Id: Context.pm,v 2.24 2001/06/29 13:09:00 abw Exp $
+# REVISION
+#   $Id: Context.pm,v 2.56 2002/04/17 14:04:38 abw Exp $
 #
 #============================================================================
 
@@ -36,7 +35,7 @@ use Template::Config;
 use Template::Constants;
 use Template::Exception;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.24 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.56 $ =~ /(\d+)\.(\d+)/);
 
 
 #========================================================================
@@ -76,7 +75,7 @@ $VERSION = sprintf("%d.%02d", q$Revision: 2.24 $ =~ /(\d+)\.(\d+)/);
 sub template {
     my ($self, $name) = @_;
     my ($prefix, $blocks, $defblocks, $provider, $template, $error);
-    my $providers;
+    my ($shortname, $providers);
 
     # references to Template::Document (or sub-class) objects objects, or
     # CODE references are assumed to be pre-compiled templates and are
@@ -85,6 +84,8 @@ sub template {
 	if UNIVERSAL::isa($name, 'Template::Document')
 	    || ref($name) eq 'CODE';
 
+    $shortname = $name;
+
     unless (ref $name) {
 	# we first look in the BLOCKS hash for a BLOCK that may have 
 	# been imported from a template (via PROCESS)
@@ -100,22 +101,18 @@ sub template {
 
 	# now it's time to ask the providers, so we look to see if any 
 	# prefix is specified to indicate the desired provider set.
-	$prefix = undef;
-
-	if ($^O =~ /win/i) {
-	    $prefix = $1	# let C:/foo through
-		if $name =~ s/^(\w{2,})://o;
+	if ($^O eq 'MSWin32') {
+	    # let C:/foo through
+	    $prefix = $1 if $shortname =~ s/^(\w{2,})://o;
 	}
 	else {
-	    $prefix = $1 
-		if $name =~ s/^(\w+)://o;
+	    $prefix = $1 if $shortname =~ s/^(\w+)://;
 	}
 
 	if (defined $prefix) {
 	    $providers = $self->{ PREFIX_MAP }->{ $prefix } 
 		|| return $self->throw(Template::Constants::ERROR_FILE,
 				      "no providers for template prefix '$prefix'");
-#	    print STDERR "prefix identified: $prefix\n";
 	}
     }
     $providers = $self->{ PREFIX_MAP }->{ default }
@@ -126,16 +123,14 @@ sub template {
     # handle references to files, text, etc., as well as templates
     # reference by name
     foreach my $provider (@$providers) {
-	($template, $error) = $provider->fetch($name);
+	($template, $error) = $provider->fetch($shortname, $prefix);
 	return $template unless $error;
-#	return $self->error($template)
 	if ($error == Template::Constants::STATUS_ERROR) {
 	    $self->throw($template) if ref $template;
 	    $self->throw(Template::Constants::ERROR_FILE, $template);
 	}
     }
 
-#    return $self->error("$name: not found");
     $self->throw(Template::Constants::ERROR_FILE, "$name: not found");
 }
 
@@ -188,9 +183,6 @@ sub filter {
 
     # request the named filter from each of the FILTERS providers in turn
     foreach my $provider (@{ $self->{ LOAD_FILTERS } }) {
-	$filter = $name, last 
-	    if ref $name;
-
 	($filter, $error) = $provider->fetch($name, $args, $self);
 	last unless $error;
 	if ($error == Template::Constants::STATUS_ERROR) {
@@ -204,9 +196,17 @@ sub filter {
     return $self->error("$name: filter not found")
 	unless $filter;
 
+    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # commented out by abw on 19 Nov 2001 to fix problem with xmlstyle
+    # plugin which may re-define a filter by calling define_filter()
+    # multiple times.  With the automatic aliasing/caching below, any
+    # new filter definition isn't seen.  Don't think this will cause
+    # any problems as filters explicitly supplied with aliases will
+    # still work as expected.
+    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # alias defaults to name if undefined
-    $alias = $name
-	unless defined($alias) or ref($name) or $args;
+    # $alias = $name
+    #     unless defined($alias) or ref($name) or $args;
 
     # cache FILTER if alias is valid
     $self->{ FILTER_CACHE }->{ $alias } = $filter
@@ -252,9 +252,6 @@ sub process {
     my ($stash, $tblocks, $error, $tmpout);
     my $output = '';
 
-#    my ($blocks, $output);
-#    my $name = $template;
-
     $template = [ $template ] unless ref $template eq 'ARRAY';
 
     # fetch compiled template for each name specified
@@ -372,51 +369,6 @@ sub include {
     return $output;
 }
 
-sub old_include {
-    my ($self, $template, $params) = @_;
-    my ($error, $blocks);
-    my $output = '';
-    my $name = $template;
-
-    # request compiled template from cache 
-    $template = $self->template($template);
-#	|| die Template::Exception->new(&Template::Constants::ERROR_FILE, 
-#			       $self->{ _ERROR } || "$template: not found" );
-
-    # localise the variable stash with any parameters passed
-    $params ||= { };
-    $params->{ component } = ref $template eq 'CODE' 
-	? { ref $name ? () : (name => $name, modtime => time()) }
-        : $template;
-    $self->{ STASH } = $self->{ STASH }->clone($params);
-
-    eval {
-	if (ref $template eq 'CODE') {
-	    $output = &$template($self);
-	}
-	elsif (ref $template) {
-	    $output = $template->process($self);
-	}
-	else {
-	    die "invalid template reference: $template\n";
-	}
-    };
-    $error = $@;
-
-    $self->{ STASH } = $self->{ STASH }->declone();
-
-    die $error if $error;
-
-    if ($self->{ TRIM }) {
-	for ($output) {
-	    s/^\s+//;
-	    s/\s+$//;
-	}
-    }
-
-    return $output;
-}
-
 
 #------------------------------------------------------------------------
 # insert($file)
@@ -426,23 +378,34 @@ sub old_include {
 
 sub insert {
     my ($self, $file) = @_;
-    my ($providers, $text, $error);
+    my ($prefix, $providers, $text, $error);
     my $output = '';
 
     my $files = ref $file eq 'ARRAY' ? $file : [ $file ];
 
     FILE: foreach $file (@$files) {
-	if ($file =~ s/^(\w{2,})://o) {
-	    $providers = $self->{ PREFIX_MAP }->{ $1 } 
-		|| return $self->throw(Template::Constants::ERROR_FILE,
-				      "no providers for file prefix '$1'");
+	my $name = $file;
+
+	if ($^O eq 'MSWin32') {
+	    # let C:/foo through
+	    $prefix = $1 if $name =~ s/^(\w{2,})://o;
 	}
 	else {
-	    $providers = $self->{ LOAD_TEMPLATES };
+	    $prefix = $1 if $name =~ s/^(\w+)://;
+	}
+
+	if (defined $prefix) {
+	    $providers = $self->{ PREFIX_MAP }->{ $prefix } 
+	    || return $self->throw(Template::Constants::ERROR_FILE,
+				   "no providers for file prefix '$prefix'");
+	}
+	else {
+	    $providers = $self->{ PREFIX_MAP }->{ default }
+	    || $self->{ LOAD_TEMPLATES };
 	}
 
 	foreach my $provider (@$providers) {
-	    ($text, $error) = $provider->load($file);
+	    ($text, $error) = $provider->load($name, $prefix);
 	    next FILE unless $error;
 	    if ($error == Template::Constants::STATUS_ERROR) {
 		$self->throw($text) if ref $text;
@@ -491,16 +454,13 @@ sub throw {
 
     # die! die! die!
     if (UNIVERSAL::isa($error, 'Template::Exception')) {
-#	print STDERR "throwing existing exception [@$error]\n";
 	die $error;
     }
     elsif (defined $info) {
-#	print STDERR "throwing new exception [$error] [$info]\n";
 	die Template::Exception->new($error, $info, $output);
     }
     else {
 	$error ||= '';
-#	print STDERR "throwing an undefined exception [$error]\n";
 	die Template::Exception->new('undef', $error, $output);
     }
 
@@ -735,7 +695,9 @@ sub _init {
     my $providers  = $self->{ LOAD_TEMPLATES };
     my $prefix_map = $self->{ PREFIX_MAP } = $config->{ PREFIX_MAP } || { };
     while (my ($key, $val) = each %$prefix_map) {
-	$prefix_map->{ $key } = [ map { $providers->[$_] } split(/\D+/, $val) ]
+	$prefix_map->{ $key } = [ ref $val ? $val :
+				  map { $providers->[$_] } 
+				  split(/\D+/, $val) ]
 	    unless ref $val eq 'ARRAY';
 #	print(STDERR "prefix $key => $val => [", 
 #	      join(', ', @{ $prefix_map->{ $key } }), "]\n");
@@ -1419,17 +1381,17 @@ L<http://www.andywardley.com/|http://www.andywardley.com/>
 
 =head1 VERSION
 
-2.24, distributed as part of the
-Template Toolkit version 2.04, released on 29 June 2001.
+2.56, distributed as part of the
+Template Toolkit version 2.07, released on 17 April 2002.
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2001 Andy Wardley.  All Rights Reserved.
-  Copyright (C) 1998-2001 Canon Research Centre Europe Ltd.
+  Copyright (C) 1996-2002 Andy Wardley.  All Rights Reserved.
+  Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Template|Template>, L<Template::Document|Template::Document>, L<Template::Exception|Template::Exception>, L<Template::Filters|Template::Filters>, L<Template::Plugins|Template::Plugins>, L<Template::Provider|Template::Provider>, L<Template::Service|Template::Service>, L<Template::Stash|Template::Stash>
+L<Template|Template>, L<Template::Document|Template::Document>, L<Template::Exception|Template::Exception>, L<Template::Filters|Template::Filters>, L<Template::Plugins|Template::Plugins>, L<Template::Provider|Template::Provider>, L<Template::Service|Template::Service>, L<Template::Stash|Template::Stash>
\ No newline at end of file
