@@ -18,7 +18,7 @@
 #
 #----------------------------------------------------------------------------
 #
-# $Id: Filters.pm,v 2.20 2001/06/29 13:09:00 abw Exp $
+# $Id: Filters.pm,v 2.52 2002/04/17 14:04:39 abw Exp $
 #
 #============================================================================
 
@@ -28,10 +28,11 @@ require 5.004;
 
 use strict;
 use base qw( Template::Base );
-use vars qw( $VERSION $DEBUG $FILTERS );
+use vars qw( $VERSION $DEBUG $FILTERS $URI_ESCAPES $PLUGIN_FILTER );
 use Template::Constants;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.20 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.52 $ =~ /(\d+)\.(\d+)/);
+
 
 #------------------------------------------------------------------------
 # standard filters, defined in one of the following forms:
@@ -45,11 +46,14 @@ $VERSION = sprintf("%d.%02d", q$Revision: 2.20 $ =~ /(\d+)\.(\d+)/);
 
 $FILTERS = {
     # static filters 
-#    'html'       => \&html_filter,
+    'uri'        => \&uri_filter,
+    'html'       => \&html_filter,
     'html_para'  => \&html_paragraph,
     'html_break' => \&html_break,
     'upper'      => sub { uc $_[0] },
     'lower'      => sub { lc $_[0] },
+    'ucfirst'    => sub { ucfirst $_[0] },
+    'lcfirst'    => sub { lcfirst $_[0] },
     'stderr'     => sub { print STDERR @_; return '' },
     'trim'       => sub { for ($_[0]) { s/^\s+//; s/\s+$// }; $_[0] },
     'null'       => sub { return '' },
@@ -57,7 +61,7 @@ $FILTERS = {
 			  $_[0] },
 
     # dynamic filters
-    'html'       => [ \&html_filter_factory,     1 ],
+    'html_entity' => [ \&html_entity_filter_factory, 1 ],
     'indent'     => [ \&indent_filter_factory,   1 ],
     'format'     => [ \&format_filter_factory,   1 ],
     'truncate'   => [ \&truncate_filter_factory, 1 ],
@@ -74,6 +78,8 @@ $FILTERS = {
     'latex'      => [ \&latex_filter_factory,    1 ],
 };
 
+# name of module implementing plugin filters
+$PLUGIN_FILTER = 'Template::Plugin::Filter';
 
 
 #========================================================================
@@ -98,11 +104,27 @@ sub fetch {
     my ($self, $name, $args, $context) = @_;
     my ($factory, $is_dynamic, $filter, $error);
 
-    # retrieve the filter factory
-    return (undef, Template::Constants::STATUS_DECLINED)
-	unless ($factory = $self->{ FILTERS }->{ $name }
-			|| $FILTERS->{ $name });
+    # allow $name to be specified as a reference to 
+    # a plugin filter object;  any other ref is 
+    # assumed to be a coderef and hence already a filter;
+    # non-refs are assumed to be regular name lookups
 
+    if (ref $name) {
+	if (UNIVERSAL::isa($name, $PLUGIN_FILTER)) {
+	    $factory = $name->factory()
+		|| return $self->error($name->error());
+	}
+	else {
+	    return $name;
+	}
+    }
+    else {
+	return (undef, Template::Constants::STATUS_DECLINED)
+	    unless ($factory = $self->{ FILTERS }->{ $name }
+		    || $FILTERS->{ $name });
+    }
+
+    # factory can be an [ $code, $dynamic ] or just $code
     if (ref $factory eq 'ARRAY') {
 	($factory, $is_dynamic) = @$factory;
     }
@@ -195,24 +217,46 @@ sub _dump {
 #========================================================================
 
 #------------------------------------------------------------------------
-# html_filter()                                         [% FILTER html %]
+# uri_filter()                                           [% FILTER uri %]
 #
-# NOTE: html filter is now dynamic to allow 'entity' option.
+# URI escape a string.  This code is borrowed from Gisle Aas' URI::Escape
+# module.  For something so simple, I can't see any validation in making
+# the user install the URI modules just for this, so we cut and paste.
 #
-# Convert any '<', '>' or '&' characters to the HTML equivalents, '&lt;',
-# '&gt;' and '&amp;', respectively.
+# URI::Escape is Copyright 1995-2000 Gisle Aas.
 #------------------------------------------------------------------------
 
-sub html_filter_not_used {
+sub uri_filter {
+    my $text = shift;
+
+    # construct and cache a lookup table for escapes (faster than
+    # doing a sprintf() for every character in every string each 
+    # time)
+    $URI_ESCAPES ||= {
+	map { ( chr($_), sprintf("%%%02X", $_) ) } (0..255),
+    };
+    
+    $text =~ s/([^;\/?:@&=+\$,A-Za-z0-9\-_.!~*'()])/$URI_ESCAPES->{$1}/g;
+    $text;
+}
+
+
+#------------------------------------------------------------------------
+# html_filter()                                         [% FILTER html %]
+#
+# Convert any '<', '>' or '&' characters to the HTML equivalents, '&lt;',
+# '&gt;' and '&amp;', respectively. 
+#------------------------------------------------------------------------
+
+sub html_filter {
     my $text = shift;
     for ($text) {
 	s/&/&amp;/g;
 	s/</&lt;/g;
 	s/>/&gt;/g;
-#	s/'/&apos;/g;
 	s/"/&quot;/g;
     }
-    $text;
+    return $text;
 }
 
 
@@ -251,32 +295,35 @@ sub html_break  {
 #========================================================================
 
 #------------------------------------------------------------------------
-# html_filter_factory(\%optins)                         [% FILTER html %]
+# html_entity_filter_factory(\%options)                 [% FILTER html %]
 #
-# Convert any '<', '>' or '&' characters to the HTML equivalents, '&lt;',
-# '&gt;' and '&amp;', respectively.  The 'entity' option can be set to
-# any true value to prevent entities of the form &word; being converted
-# to &amp;word;
+# Dynamic version of the static html filter which attempts to locate the
+# Apache::Util or HTML::Entities modules to perform full entity encoding
+# of the text passed.  Returns an exception if one or other of the 
+# modules can't be located.
 #------------------------------------------------------------------------
 
-sub html_filter_factory {
+sub html_entity_filter_factory {
     my $context = shift;
-    my $opts = @_ && ref $_[-1] eq 'HASH' ? pop @_ : { };
-    return sub {
-	my $text = shift;
-	for ($text) {
-	    if ($opts->{ entity }) {
-		s/&(?!\w+;)/&amp;/g;
-	    }
-	    else {
-		s/&/&amp;/g;
-	    }
-	    s/</&lt;/g;
-	    s/>/&gt;/g;
-	    s/"/&quot;/g;
-	}
-	return $text;
+
+    # if Apache::Util is installed then we use it
+    eval { 
+	require Apache::Util;
+        Apache::Util::escape_html('');
     };
+    return \&Apache::Util::escape_html
+	unless $@;
+
+    # otherwise if HTML::Entities is installed then we use that
+    eval {
+	require HTML::Entities;
+    };
+    return \&HTML::Entities::encode_entities
+	unless $@;
+
+    return (undef, Template::Exception->new( html_all => 
+      	            'cannot locate Apache::Util or HTML::Entities' ));
+
 }
 
 
@@ -562,7 +609,7 @@ sub latex_filter_factory
             # do on other OSs
             #
             my $LaTeX_arg = "\\nonstopmode\\input{$texDoc}";
-            $LaTeX_arg = "'$LaTeX_arg'" if ( $^O !~ /win/i );
+            $LaTeX_arg = "'$LaTeX_arg'" if ( $^O ne 'MSWin32' );
             if ( system("$LaTeXPath $LaTeX_arg"
                    . " 1>$devnull 2>$devnull 0<$devnull") ) {
                 my $texErrs = "";
@@ -878,6 +925,26 @@ output:
 
     hello world
 
+=head2 ucfirst
+
+Folds the first character of the input to UPPER CASE.
+
+    [% "hello" | FILTER ucfirst %]
+
+output:
+
+    Hello
+
+=head2 lcfirst
+
+Folds the first character of the input to lower case.
+
+    [% "HELLO" | FILTER lcfirst %]
+
+output:
+
+    hELLO
+
 =head2 trim
 
 Trims any leading or trailing whitespace from the input text.  Particularly 
@@ -909,7 +976,7 @@ output:
 =head2 html
 
 Converts the characters 'E<lt>', 'E<gt>' and '&' to '&lt;', '&gt;' and
-'&amp', respectively, protecting them from being interpreted as
+'&amp;', respectively, protecting them from being interpreted as
 representing HTML tags or entities.
 
     [% FILTER html %]
@@ -920,6 +987,23 @@ output:
 
     Binary "&lt;=&gt;" returns -1, 0, or 1 depending on...
 
+=head2 html_entity
+
+The html filter is fast and simple but it doesn't encode the full
+range of HTML entities that your text may contain.  The html_entity
+filter uses either the Apache::Util module (which is written in C and
+is therefore faster) or the HTML::Entities module (written in Perl but
+equally as comprehensive) to perform the encoding.  If one or other of
+these modules are installed on your system then the text will be
+encoded (via the escape_html() or encode_entities() subroutines
+respectively) to convert all extended characters into their
+appropriate HTML entities (e.g. converting '' to '&eacute;').  If
+neither module is available on your system then an 'html_all' exception
+will be thrown reporting an appropriate message.   
+
+For further information on HTML entity encoding, see
+http://www.w3.org/TR/REC-html40/sgml/entities.html.
+
 =head2 html_para
 
 This filter formats a block of text into HTML paragraphs.  A sequence of 
@@ -960,6 +1044,25 @@ output:
     <br>
     Mary had a little lamb.
 
+=head2 uri
+
+This filter URI escapes the input text, converting any characters 
+outside of the permitted URI character set (as defined by RFC 2396)
+into a C<%nn> hex escape.
+
+    [% 'my file.html' | uri %]
+
+output:
+
+    my%20file.html
+
+Note that URI escaping isn't always enough when generating hyperlinks in
+an HTML document.  The C<&> character, for example, is valid in a URI and
+will not be escaped by the URI filter.  In this case you should also filter
+the text through the 'html' filter.
+
+    <a href="[% filename | uri | html %]">click here</a>
+
 =head2 indent(pad)
 
 Indents the text block by a fixed pad string or width.  The 'pad' argument
@@ -1208,17 +1311,17 @@ L<http://www.andywardley.com/|http://www.andywardley.com/>
 
 =head1 VERSION
 
-2.20, distributed as part of the
-Template Toolkit version 2.04, released on 29 June 2001.
+2.52, distributed as part of the
+Template Toolkit version 2.07, released on 17 April 2002.
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2001 Andy Wardley.  All Rights Reserved.
-  Copyright (C) 1998-2001 Canon Research Centre Europe Ltd.
+  Copyright (C) 1996-2002 Andy Wardley.  All Rights Reserved.
+  Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Template|Template>, L<Template::Context|Template::Context>
+L<Template|Template>, L<Template::Context|Template::Context>, L<Template::Manual::Filters|Template::Manual::Filters>
\ No newline at end of file
