@@ -31,7 +31,7 @@
 # 
 #----------------------------------------------------------------------------
 #
-# $Id: Parser.pm,v 2.22 2001/06/29 13:09:00 abw Exp $
+# $Id: Parser.pm,v 2.53 2002/04/17 14:04:39 abw Exp $
 #
 #============================================================================
 
@@ -42,7 +42,7 @@ require 5.004;
 use strict;
 use vars qw( $VERSION $DEBUG $ERROR );
 use base qw( Template::Base );
-use vars qw( $TAG_STYLE $DEFAULT_STYLE );
+use vars qw( $TAG_STYLE $DEFAULT_STYLE $QUOTED_ESCAPES );
 
 use Template::Constants qw( :status :chomp );
 use Template::Directive;
@@ -54,7 +54,7 @@ use constant ACCEPT   => 1;
 use constant ERROR    => 2;
 use constant ABORT    => 3;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.22 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.53 $ =~ /(\d+)\.(\d+)/);
 $DEBUG   = 0 unless defined $DEBUG;
 $ERROR   = '';
 
@@ -88,6 +88,12 @@ $DEFAULT_STYLE = {
     EVAL_PERL   => 0,
 };
 
+$QUOTED_ESCAPES = {
+	n => "\n",
+	r => "\r",
+	t => "\t",
+};
+
 
 #========================================================================
 #                      -----  PUBLIC METHODS -----
@@ -117,6 +123,7 @@ sub new {
 	EVAL_PERL   => 0,
 	GRAMMAR     => undef,
 	_ERROR      => '',
+	FACTORY     => 'Template::Directive',
     }, $class;
 
     # update self with any relevant keys in config
@@ -128,7 +135,6 @@ sub new {
 	require Template::Grammar;
 	Template::Grammar->new();
     };
-    $self->{ FACTORY } ||= 'Template::Directive';
 
 #    # determine START_TAG and END_TAG for specified (or default) TAG_STYLE
 #    $tagstyle = $self->{ TAG_STYLE } || 'default';
@@ -323,7 +329,7 @@ sub split_text {
 	# and now the directive, along with line number information
 	if (length $dir) {
 	    # the TAGS directive is a compile-time switch
-	    if ($dir =~ /TAGS\s+(.*)/i) {
+	    if ($dir =~ /^TAGS\s+(.*)/i) {
 		my @tags = split(/\s+/, $1);
 		if (scalar @tags > 1) {
 		    ($start, $end) = map { quotemeta($_) } @tags;
@@ -374,10 +380,10 @@ sub interpolate_text {
     my ($pre, $var, $dir);
 
 
-    while ($text =~ 
-	   /
-	   ( (?: \\. | [^\$] )+ )   # escaped or non-'$' character [$1]
-	   | 
+   while ($text =~
+           /
+           ( (?: \\. | [^\$] ){1,4000} ) # escaped or non-'$' character [$1]
+           |
 	   ( \$ (?:		    # embedded variable	           [$2]
 	     (?: \{ ([^\}]*) \} )   # ${ ... }                     [$3]
 	     |
@@ -385,7 +391,7 @@ sub interpolate_text {
 	     )
 	   )
 	/gx) {
-    
+
 	($pre, $var, $dir) = ($1, $3 || $4, $2);
 
 	# preceding text
@@ -495,11 +501,13 @@ sub tokenise_directive {
 	        if ($token =~ /[\$\\]/) {
 		    $type = 'QUOTED';
 		    # unescape " and \ but leave \$ escaped so that 
-		    # interpolate_text() doesn't incorrectly treat it
+			# interpolate_text() doesn't incorrectly treat it
 		    # as a variable reference
 #		    $token =~ s/\\([\\"])/$1/g;
-		    $token =~ s/\\([^\$n])/$1/g;
-		    $token =~ s/\\n/\n/g;
+			for ($token) {
+				s/\\([^\$nrt])/$1/g;
+				s/\\([nrt])/$QUOTED_ESCAPES->{ $1 }/ge;
+			}
 		    push(@tokens, ('"') x 2,
 				  @{ $self->interpolate_text($token) },
 				  ('"') x 2);
@@ -1268,15 +1276,15 @@ L<http://www.andywardley.com/|http://www.andywardley.com/>
 
 =head1 VERSION
 
-2.22, distributed as part of the
-Template Toolkit version 2.04, released on 29 June 2001.
+2.53, distributed as part of the
+Template Toolkit version 2.07, released on 17 April 2002.
 
  
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2001 Andy Wardley.  All Rights Reserved.
-  Copyright (C) 1998-2001 Canon Research Centre Europe Ltd.
+  Copyright (C) 1996-2002 Andy Wardley.  All Rights Reserved.
+  Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
