@@ -18,7 +18,7 @@
 #
 #----------------------------------------------------------------------------
 #
-# $Id: Table.pm,v 2.31 2001/11/06 15:00:20 abw Exp $
+# $Id: Table.pm,v 2.46 2002/04/17 14:04:46 abw Exp $
 #
 #============================================================================
 
@@ -31,7 +31,7 @@ use vars qw( @ISA $VERSION $AUTOLOAD );
 use base qw( Template::Plugin );
 use Template::Plugin;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.31 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.46 $ =~ /(\d+)\.(\d+)/);
 
 
 #------------------------------------------------------------------------
@@ -54,16 +54,16 @@ sub new {
     # the data it contains
     if (UNIVERSAL::isa($data, 'Template::Iterator')) {
 	($data, $error) = $data->get_all();
-	return $class->fail("iterator failed to provide data for table: ",
-			    $error)
+	return $class->error("iterator failed to provide data for table: ",
+			     $error)
 	    if $error;
     }
 	
-    return $class->fail('invalid table data, expecting a list')
+    return $class->error('invalid table data, expecting a list')
 	unless ref $data eq 'ARRAY';
 
     $params ||= { };
-    return $class->fail('invalid table parameters, expecting a hash')
+    return $class->error('invalid table parameters, expecting a hash')
 	unless ref $params eq 'HASH';
 
     # ensure keys are folded to upper case
@@ -394,6 +394,21 @@ the data across.
     .  .
     .
 
+Example code to do so would be much like the following:
+
+    [% USE table(alphabet, rows=3) %]
+    [% FOREACH cols = table.cols %]
+      [% FOREACH item = cols %]
+        [% item %]
+      [% END %]
+    [% END %]
+
+    a  b  c
+    d  e  f
+    g  h  i
+    j  .  .
+    .
+
 In addition to a list reference, the Table plugin constructor may be 
 passed a reference to a Template::Iterator object or subclass thereof.
 The get_all() method is first called on the iterator to return all 
@@ -423,17 +438,17 @@ L<http://www.andywardley.com/|http://www.andywardley.com/>
 
 =head1 VERSION
 
-2.31, distributed as part of the
-Template Toolkit version 2.06, released on 07 November 2001.
+2.46, distributed as part of the
+Template Toolkit version 2.07, released on 17 April 2002.
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2001 Andy Wardley.  All Rights Reserved.
-  Copyright (C) 1998-2001 Canon Research Centre Europe Ltd.
+  Copyright (C) 1996-2002 Andy Wardley.  All Rights Reserved.
+  Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Template::Plugin|Template::Plugin>
+L<Template::Plugin|Template::Plugin>
\ No newline at end of file
